#include "kanji_union.h"

namespace {
    const int READING = false;
    const int WRITING = true;
}

bool openClose(char *file_name, bool write) {
    using namespace std;

    bool b = true;
    if(write) {
        ofstream out(file_name);
        if( out.fail() ) {
            b = false;
            strcpy(file_name, MS932_SJIS_TEXT);
        }

    } else {
        ifstream in(file_name);
        if( in.fail() ) {
            b = false;
            errorHandler("ifstream in() error:", file_name);
            exit(1);
        }

    }

    return b;
}

int Regulate::readingToProduceSJIS(char *new_name) {
    using namespace std;

    bool kanji = false;
    char *p;
    char a[3] = {0, 0, 0};
    char file_name[ FILE_NAME_SIZE + 1 ];
    char newLine[3] = {0x0A, 0, 0};
    int c;
    int exit_code = 0;
    stringstream *os = new stringstream();
    unsigned char uc;
    unsigned char firstByte = 0x40;

    strcpy(file_name, documentName.c_str() );
    if(! openClose(file_name, READING) ) {
        cerr << "openClose() error-1: " << file_name << endl;
        return ERROR_CODE;
    }

    if(! openClose(new_name, WRITING) ) {
        cerr << "openClose() error-2: " << new_name << endl;
        return ERROR_CODE;
    }

    ifstream in(file_name);
    ofstream sjisFile(new_name, ios::out | ios::binary| ios::trunc );
    SparseSJIS *sjis = new SparseSJIS();
    sjis->clear();
    if(! sjis->setting(no_searching, &whichTable) ) {
        errorHandler("readingToProduceSJIS() error:", "sjis->setting(no_searching, &whichTable)");
        return 1;
    }

    if(carriage_return) {
        newLine[0] = 0x0D;
        newLine[1] = 0x0A;
    }

    while( in.getline(buffer, BUFFER_SIZE) ) {
        delete os;
        os = new stringstream();
        for(int i = 0; buffer[i]; ++i) {
            uc = (unsigned char)buffer[i];
            c = (int)uc;

            if(kanji) {
                a[0] = (char)firstByte;
                a[1] = buffer[i];
                *os << sjis->working( inverted(a) );
                kanji = false;
                continue;
            }

            if(0x81 <= c && c<= 0xFC) {
                if(KANA_KUTEN <= c && c <= KANA_HANDAKUTEN) {
                    *os << sjis->working(c);
                    kanji = false;
                } else {
                    firstByte = uc;
                    kanji = true;
                }

            } else {
                *os << uc;
                kanji = false;
            }

        }

        sjisFile << os->str() << newLine;
    }

    delete os;
    delete sjis;
    return exit_code;
}

int produceBinaryFile(const char *name, bool no_removing) {
    using namespace std;

    char c[BLOCK_SIZE];
    FILE *ancient = fopen(name, "rb");
    int exit_code = 0;
    int size;
    FILE *temporary = fopen(TEMPORARY_BINARY, "wb");
    if(temporary == NULL) {
        cerr << "fopen() error: " << TEMPORARY_BINARY << endl;
        exit(1);
    }

    if(ancient == NULL) {
        exit_code = FILE_ERROR_CODE;
    } else {
        do {
            size = fread(c, 1, BLOCK_SIZE, ancient);
            fwrite(c, size, 1, temporary);
        } while(size == BLOCK_SIZE);

        fclose(temporary);
        fclose(ancient);

        if(! no_removing) {
            remove(name);
        }

    }

    return exit_code;
}

int renameBinaryFile(const char *name, bool no_removing) {
    char c[BLOCK_SIZE];
    int exit_code = 0;
    int size;
    FILE *temporary = fopen(TEMPORARY_BINARY, "rb");
    FILE *recent = fopen(name, "wb");
    if(recent == NULL) {
        recent = fopen(MS932_SJIS_TEXT, "wb");
    }

    if(temporary == NULL) {
        exit_code = FILE_ERROR_CODE;
    } else {
        do {
            size = fread(c, 1, BLOCK_SIZE, temporary);
            fwrite(c, size, 1, recent);
        } while(size == BLOCK_SIZE);

        fclose(recent);
        fclose(temporary);

        if(! no_removing) {
            remove(TEMPORARY_BINARY);
        }

    }

    return exit_code;
}

