#include "kanji_union.h"

/*
dayu Version 1.215
Editing MS932~Shift_JIS.bin and regulating a Shift-JIS file
Copyright 2019 Sogaya

gcc (GCC) 3.4.2 (mingw-special)
Digital Mars C/C++ Compiler Version 8.57, STLport 4.5.3
Borland C++ 5.5.1 for Win32
Embarcadero C++ 7.30 for Win32

dayu assumes that the little-endian architecture underlies operating systems (Linux, Windows).
*/

char binaryName[ FILE_NAME_SIZE + 1 ];
char *binary_name = & binaryName[0];

int main(int argc, char *argv[]) {
    using namespace std;

    const bool REMOVE_ANCIENT_FILE = false;
    const bool WITH_COMMENTS = true;
    const bool WITHOUT_COMMENTS = false;
    const char SETTING_TEXT[] = "twenty.txt";
    const int ASCII_SIZE = 1;
    const int ASCII_TERM_SIZE = 2;
    const int ERROR_CODE = 1;
    const int KATAKANA_TERM_SIZE = 2;
    const int MS932_TERM_SIZE = 2;
    const int SJIS_TERM_SIZE = 2;
    char ASCII[] = "-a";
    char DIFFERENT_TIME[] = "-t-";
    char DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE[] = "-w-";
    char EDIT_TABLE[] = "-e";
    char KATAKANA[] = "-k";
    char LIST_FILE[] = "-f";
    char MS932[] = "-m";
    char NO_SKIPPING[] = "-i-";
    char READ_DOCUMENT[] = "-r";
    char SEARCH_TABLE[] = "-d-";
    char SHIFT_JIS[] = "-s";
    char VERSION_COMMENTS[] = "-VC";
    char VERSION_GUIDE[] = "-V";

    bool carriage_return = true;
    bool document_to_be_read = true;
    bool list_to_be_read = false;
    bool edit_table = true;
    bool identical_time = false;
    bool no_reading = false;
    bool no_removing = false;
    bool no_searching = false;
    bool produce_binary_file = false;
    bool result = false;
    bool skip_control_characters = false;
    bool which_table = false;
    char *edge;
    char *p;
    char ms932_text[ FILE_NAME_SIZE + 1 ];
    char normal[NORMAL_ARRAY_SIZE] = {0x87, 0x53, 0}; /* S */
    char sjis_text[ FILE_NAME_SIZE + 1 ];
    char substitute[SUBSTITUTE_ARRAY_SIZE] = {0x93, 0xF9, 0}; /*  */
    char text_of_substitutes_list[ FILE_NAME_SIZE + 1 ];
    int exit_code = 0;
    int file_name_size;
    int size;

    memset(ms932_text, 0, FILE_NAME_SIZE + 1 );
    memset(text_of_substitutes_list, 0,  FILE_NAME_SIZE + 1 );

    binary_name[FILE_NAME_SIZE] = '\0';
    strncpy(binary_name, MS932_SJIS_BIN, FILE_NAME_SIZE);

    sjis_text[FILE_NAME_SIZE] = '\0';
    strncpy(sjis_text, MS932_SJIS_TEXT, FILE_NAME_SIZE);

// Acquiring parameters from SETTING_TEXT unless the user removes it.
    acquire_parameters_from_text(SETTING_TEXT, binary_name, ms932_text, sjis_text, normal, substitute, text_of_substitutes_list, &produce_binary_file, &carriage_return, &no_searching, &edit_table, &skip_control_characters, &no_reading, &identical_time, &which_table, &no_removing);

    for(int i = 1; i < argc; ++i) {
        if(! strcmp(argv[i], CARRIAGE_RETURN_LINE_FEED) ) {
            carriage_return = true;
        } else if(! strcmp(argv[i], DIFFERENT_TIME) ) {
            identical_time = false;
        } else if(! strcmp(argv[i], DO_NOT_REMOVE_TEMPORARY_BINARY_FILE) ) {
            no_removing = true;
        } else if(! strcmp(argv[i], DO_NOT_SEARCH_TABLE) ) {
            no_searching = true;
        } else if(! strcmp(argv[i], DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE) ) {
            which_table = false;
        } else if(! strcmp(argv[i], EDIT_TABLE) ) {
            edit_table = true;
        } else if(! strcmp(argv[i], IDENTICAL_TIME) ) {
            identical_time = true;
        } else if(! strcmp(argv[i], LINE_FEED) ) {
            carriage_return = false;
        } else if(! strcmp(argv[i], LIST_FILE) ) {
            list_to_be_read = true;
        } else if(! strcmp(argv[i], NO_EDITING) ) {
            edit_table = false;
        } else if(! strcmp(argv[i], NO_READING) ) {
            no_reading = true;
        } else if(! strcmp(argv[i], NO_SKIPPING) ) {
            skip_control_characters = false;
        } else if(! strcmp(argv[i], READ_DOCUMENT) ) {
            no_reading = false;
        } else if(! strcmp(argv[i], SEARCH_TABLE) ) {
            no_searching = false;
        } else if(! strcmp(argv[i], SKIPPING) ) {
            skip_control_characters = true;
        } else if(! strcmp(argv[i], TEMPORARY_BINARY_FILE) ) {
            produce_binary_file = true;
        } else if(! strcmp(argv[i], VERSION_COMMENTS) ) {
            versionGuide(VERSION_NUMBER, WITH_COMMENTS);
        } else if(! strcmp(argv[i], VERSION_GUIDE) ) {
            versionGuide(VERSION_NUMBER, WITHOUT_COMMENTS);
        } else if(! strcmp(argv[i], WHICH_TABLE) ) {
            which_table = true;
        } else {
            p = argv[i];
            edge = & p[0];

            p = strstr(argv[i], ASCII);
            if(p == edge) {
                memset(substitute, 0, SUBSTITUTE_ARRAY_SIZE);
                p = & argv[i][ASCII_TERM_SIZE];
                memcpy(substitute, p, ASCII_SIZE);
                continue;
            }

            p = strstr(argv[i], SHIFT_JIS);
            if(p == edge) {
                memset(substitute, 0, SUBSTITUTE_ARRAY_SIZE);
                p = & argv[i][SJIS_TERM_SIZE];
                memcpy(substitute, p, SJIS_KANJI_SIZE);
                continue;
            }

            p = strstr(argv[i], KATAKANA);
            if(p == edge) {
                memset(normal, 0, NORMAL_ARRAY_SIZE);
                p = & argv[i][KATAKANA_TERM_SIZE];
                memcpy(normal, p, KATAKANA_SIZE);
                continue;
            }

            p = strstr(argv[i], MS932);
            if(p == edge) {
                memset(normal, 0, NORMAL_ARRAY_SIZE);
                p = & argv[i][MS932_TERM_SIZE];
                memcpy(normal, p, SJIS_KANJI_SIZE);
                continue;
            }

            if(list_to_be_read) {
                text_of_substitutes_list[FILE_NAME_SIZE] = '\0';
                strncpy(text_of_substitutes_list, argv[i], FILE_NAME_SIZE);
                list_to_be_read = false;
            } else {
                if(document_to_be_read) {
                    ms932_text[FILE_NAME_SIZE] = '\0';
                    strncpy(ms932_text, argv[i], FILE_NAME_SIZE);
                    document_to_be_read = false;
                } else {
                    sjis_text[FILE_NAME_SIZE] = '\0';
                    strncpy(sjis_text, argv[i], FILE_NAME_SIZE);
                }

            }

        } // else

    } // for(int i = 1; i < argc; ++i)

    if(edit_table) {
        string b(binary_name);
        string n(normal);
        string s(substitute);
        string text(text_of_substitutes_list);

        size = strlen(text_of_substitutes_list);
        if(size == 0) {
            result = editRecord(n, s, b, no_searching, &which_table);
        } else {
            result = editManyRecords(text, b, no_searching, &which_table);
        }

        if(result) {
            if(no_reading) {
                return 0;
            }

        } else {
            return ERROR_CODE;
        }

    }

    file_name_size = strlen(ms932_text);
    if(file_name_size == 0) {
        Standard_input *s = new Standard_input(no_searching, which_table);
        if(skip_control_characters) {
            exit_code = s->standard_input_output_skipping();
        } else {
            exit_code = s->standard_input_output();
        }

        delete s;
    } else {
        if(! openClose(ms932_text) ) {
            cerr << "openClose() error: " << ms932_text << endl;
        }

        Regulate *r = new Regulate(ms932_text, carriage_return, no_searching, which_table);

        if(identical_time) {
            produceBinaryFile(ms932_text, REMOVE_ANCIENT_FILE);
            renameBinaryFile(ms932_text, no_removing);
        }

        if(produce_binary_file) {
            if(skip_control_characters) {
                exit_code = r->skippingControlCharactersToProduceSJIS(TEMPORARY_BINARY);
            } else {
                exit_code = r->readingToProduceSJIS(TEMPORARY_BINARY);
            }

            renameBinaryFile(sjis_text, no_removing);
        } else {
            if(skip_control_characters) {
                exit_code = r->skippingControlCharactersToProduceSJIS(sjis_text);
            } else {
                exit_code = r->readingToProduceSJIS(sjis_text);
            }

        }

        delete r;
    }

    return exit_code;
}

/*
1:  binary_name                       MS932~Shift_JIS.bin
2:  document_name                     KeyWords.csv
3:  new_name                          dayu.txt
4:  normal                            S
5:  substitute                        
6:  text_of_substitutes_list          bars.txt
7:  TEMPORARY_BINARY_FILE             -b
8:  CARRIAGE_RETURN_LINE_FEED         -CRLF
9:  DO_NOT_SEARCH_TABLE               -d
10: NO_EDITING                        -e-
11: SKIPPING                          -i
12: NO_READING                        -r-
13: IDENTICAL_TIME                    -t
14: WHICH_TABLE                       -w
15: DO_NOT_REMOVE_TEMPORARY_BINARY    -y
*/

void acquire_parameters_from_text(const char *parameters_text, char *binary_name, char *document_name, char *new_name, char *normal, char *substitute, char *text_of_substitutes_list, bool *produce_binary_file, bool *carriage_return, bool *no_searching, bool *edit_table, bool *skip_control_characters, bool *no_reading, bool *identical_time, bool *which_table, bool *no_removing) {
    char *p;
    char c[C_SIZE];
    FILE *setting = fopen(parameters_text, "rt");
    int k;
    int line_number = 0;

    if(setting != NULL) {
        for(;;) {
            p = fgets(c, C_SIZE, setting);
            if(p == NULL) {
                break;
            } else {
                k = strcspn(c, "\n',|\t");
                c[k] = '\0';
            }

            line_number += 1;
            switch(line_number) {
            case 1:
                strcpy(binary_name, c); // MS932~Shift_JIS.bin
                break;
            case 2:
                strcpy(document_name, c); // KeyWords.csv
                break;
            case 3:
                strcpy(new_name, c); // dayu.txt
                break;
            case 4:
                strcpy(normal, c); // S
                break;
            case 5:
                strcpy(substitute, c); // 
                break;
            case 6:
                strcpy(text_of_substitutes_list, c); // bars.txt
                break;
            case 7:
                if(! strcmp(TEMPORARY_BINARY_FILE, c) ) { // -b
                    *produce_binary_file = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *produce_binary_file = false;
                } else {
                    *produce_binary_file = true;
                }

                break;
            case 8:
                if(! strcmp(CARRIAGE_RETURN_LINE_FEED, c) ) { // -CRLF
                    *carriage_return = true;
                } else if(! strcmp(LINE_FEED, c) ) { // -LF
                    *carriage_return = false;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *carriage_return = false;
                } else {
                    *carriage_return = true;
                }

                break;
            case 9:
                if(! strcmp(DO_NOT_SEARCH_TABLE, c) ) { // -d
                    *no_searching = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *no_searching = false;
                } else {
                    *no_searching = true;
                }

                break;
            case 10:
                if(! strcmp(NO_EDITING, c) ) { // -e-
                    *edit_table = false;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *edit_table = true;
                } else {
                    *edit_table = false;
                }

                break;
            case 11:
                if(! strcmp(SKIPPING, c) ) { // -i
                    *skip_control_characters = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *skip_control_characters = false;
                } else {
                    *skip_control_characters = true;
                }

                break;
            case 12:
                if(! strcmp(NO_READING, c) ) { // -r-
                    *no_reading = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *no_reading = false;
                } else {
                    *no_reading = true;
                }

                break;
            case 13:
                if(! strcmp(IDENTICAL_TIME, c) ) { // -t
                    *identical_time = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *identical_time = false;
                } else {
                    *identical_time = true;
                }

                break;
            case 14:
                if(! strcmp(WHICH_TABLE, c) ) { // -w
                    *which_table = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *which_table = false;
                } else {
                    *which_table = true;
                }

                break;
            case 15:
                if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, c) ) { // -y
                    *no_removing = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *no_removing = false;
                } else {
                    *no_removing = true;
                }

                break;
            default:
                break;
            } // switch(line_number)

        } // for(;;)

        fclose(setting);
    } // if(setting != NULL)

}
