#include "kanji_union.h"

namespace {
    const int READING = false;
    const int WRITING = true;
}

int BELL_ASCII_KATAKANA[BELL_ASCII_KATAKANA_COUNT] = {
    0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
    0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
    0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 
    0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 
    0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 
    0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 
    0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 
    0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x07, 
    0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
    0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
    0x07, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 
    0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 
    0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 
    0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF
};

int Regulate::skippingControlCharactersToProduceSJIS(char *new_name) {
    using namespace std;

    bool kanji = false;
    char *p;
    char a[3] = {0, 0, 0};
    char file_name[ FILE_NAME_SIZE + 1 ];
    char newLine[3] = {0x0A, 0, 0};
    int c;
    int exit_code = 0;
    stringstream *os = new stringstream();
    unsigned char uc;
    unsigned char firstByte = 0x40;

    strcpy(file_name, documentName.c_str() );
    if(! openClose(file_name, READING) ) {
        cerr << "openClose() error-1: " << file_name << endl;
        return ERROR_CODE;
    }

    if(! openClose(new_name, WRITING) ) {
        cerr << "openClose() error-2: " << new_name << endl;
        return ERROR_CODE;
    }

    ifstream in(file_name);
    ofstream sjisFile(new_name, ios::out | ios::binary| ios::trunc );
    SparseSJIS *sjis = new SparseSJIS();
    sjis->clear();
    if(! sjis->setting(no_searching, &whichTable) ) {
        errorHandler("skippingControlCharactersToProduceSJIS() error:", "sjis->setting(no_searching, &whichTable)");
        return 1;
    }

    if(carriage_return) {
        newLine[0] = 0x0D;
        newLine[1] = 0x0A;
    }

    while( in.getline(buffer, BUFFER_SIZE) ) {
        delete os;
        os = new stringstream();
        for(int i = 0; buffer[i]; ++i) {
            uc = (unsigned char)buffer[i];
            c = (int)uc;

            if(kanji) {
                a[0] = (char)firstByte;
                a[1] = buffer[i];
                *os << sjis->working( inverted(a) );
                kanji = false;
                continue;
            }

            if(0x81 <= c && c<= 0xFC) {
                if(KANA_KUTEN <= c && c <= KANA_HANDAKUTEN) {
                    *os << sjis->working(c);
                    kanji = false;
                } else {
                    firstByte = uc;
                    kanji = true;
                }

            } else {
                c = BELL_ASCII_KATAKANA[c];
                if(c == BELL) {
                    continue; // skipping control characters
                } else {
                    *os << uc;
                }

                kanji = false;
            }

        }

        sjisFile << os->str() << newLine;
    }

    delete os;
    delete sjis;
    return exit_code;
}

int Standard_input::standard_input_output_skipping() {
    using namespace std;

    bool kanji = false;
    char *p;
    char a[3] = {0, 0, 0};
    char newLine[3] = {0x0D, 0x0A, 0};
    int c;
    int exit_code = 0;
    unsigned char uc;
    unsigned char firstByte = 0x40;

    SparseSJIS *sjis = new SparseSJIS();
    sjis->clear();
    if(! sjis->setting(no_searching, &whichTable) ) {
        errorHandler("standard_input_output_skipping() error:", "sjis->setting(no_searching, &whichTable)");
        return ERROR_CODE;
    }

    while( cin.getline(buffer, BUFFER_SIZE) ) {
        for(int i = 0; buffer[i]; ++i) {
            uc = (unsigned char)buffer[i];
            c = (int)uc;

            if(kanji) {
                a[0] = (char)firstByte;
                a[1] = buffer[i];
                cout << sjis->working( inverted(a) );
                kanji = false;
                continue;
            }

            if(0x81 <= c && c<= 0xFC) {
                if(KANA_KUTEN <= c && c <= KANA_HANDAKUTEN) {
                    cout << sjis->working(c);
                    kanji = false;
                } else {
                    firstByte = uc;
                    kanji = true;
                }

            } else {
                c = BELL_ASCII_KATAKANA[c];
                if(c == BELL) {
                    continue; // skipping control characters
                } else {
                    cout << uc;
                }

                kanji = false;
            }

        }

        cout << endl;
    }

    delete sjis;
    return exit_code;
}
