#include "kanji_union.h"

/*
diyo C++ Version 1.16
Editing MS932~EUC-JP.bin
Copyright 2019 Sogaya

gcc (GCC) 3.4.2 (mingw-special)
C++Builder 5
Borland C++ 5.5.1 for Win32
Digital Mars Compiler Version 8.42n (Digital Mars C/C++ Compiler Version 8.57)
Embarcadero C++ 7.30 for Win32

diyo assumes that the little-endian architecture underlies operating systems (Linux, Windows).
*/

int main(int argc, char *argv[]) {
    using namespace std;

    const bool WITH_COMMENTS = true;
    const bool WITHOUT_COMMENTS = false;
    const int ASCII_SIZE = 1;
    const int ASCII_TERM_SIZE = 2;
    const int ERROR_CODE = 1;
    const int EUC_TERM_SIZE = 2;
    const int SJIS_TERM_SIZE = 2;
    const int KATAKANA_TERM_SIZE = 2;
    char ASCII[] = "-a";
    char DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE[] = "-w-";
    char EUC_JP[] = "-e";
    char KATAKANA[] = "-k";
    char LIST_FILE[] = "-f";
    char SEARCH_TABLE[] = "-d-";
    char SHIFT_JIS[] = "-s";
    char VERSION_COMMENTS[] = "-VC";
    char VERSION_GUIDE[] = "-V";

    bool no_searching = false;
    bool file_to_be_read = false;
    bool result = true;
    bool which_table = false;
    char *edge;
    char *p;
    char binary_name[ FILE_NAME_SIZE + 1 ];
    char normal[NORMAL_ARRAY_SIZE] = {0x87, 0x53, 0}; // S
    char text_of_substitutes_list[ FILE_NAME_SIZE + 1 ];
    char substitute[SUBSTITUTE_ARRAY_SIZE] = {0x93, 0xF9, 0}; // 
    int exit_code, size;
    memset(text_of_substitutes_list, 0,  FILE_NAME_SIZE + 1 );
    strcpy(binary_name, SJIS_EUC_BIN);

// Acquiring parameters from SETTING_TEXT unless the user removes it.
    acquire_parameters_from_text(SETTING_TEXT, binary_name, normal, substitute, text_of_substitutes_list, &no_searching, &which_table);

    for(int i = 1; i < argc; ++i) {
        if(! strcmp(argv[i], DO_NOT_SEARCH_TABLE) ) {
            no_searching = true;
        } else if(! strcmp(argv[i], DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE) ) {
            which_table = false;
        } else if(! strcmp(argv[i], LIST_FILE) ) {
            file_to_be_read = true;
        } else if(! strcmp(argv[i], SEARCH_TABLE) ) {
            no_searching = false;
        } else if(! strcmp(argv[i], VERSION_COMMENTS) ) {
            versionGuide(VERSION_NUMBER, WITH_COMMENTS);
        } else if(! strcmp(argv[i], VERSION_GUIDE) ) {
            versionGuide(VERSION_NUMBER, WITHOUT_COMMENTS);
        } else if(! strcmp(argv[i], WHICH_TABLE) ) {
            which_table = true;
        } else {
            p = argv[i];
            edge = & p[0];

            p = strstr(argv[i], ASCII);
            if(p == edge) {
                memset(substitute, 0, SUBSTITUTE_ARRAY_SIZE);
                p = & argv[i][ASCII_TERM_SIZE];
                memcpy(substitute, p, ASCII_SIZE);
                continue;
            }

            p = strstr(argv[i], EUC_JP);
            if(p == edge) {
                memset(substitute, 0, SUBSTITUTE_ARRAY_SIZE);
                p = & argv[i][EUC_TERM_SIZE];
                memcpy(substitute, p, EUC_KANJI_SIZE);
                continue;
            }

            p = strstr(argv[i], KATAKANA);
            if(p == edge) {
                memset(normal, 0, NORMAL_ARRAY_SIZE);
                p = & argv[i][KATAKANA_TERM_SIZE];
                memcpy(normal, p, KATAKANA_SIZE);
                continue;
            }

            p = strstr(argv[i], SHIFT_JIS);
            if(p == edge) {
                memset(normal, 0, NORMAL_ARRAY_SIZE);
                p = & argv[i][SJIS_TERM_SIZE];
                memcpy(normal, p, SJIS_KANJI_SIZE);
                continue;
            }

            if(file_to_be_read) {
                text_of_substitutes_list[FILE_NAME_SIZE] = '\0';
                strncpy(text_of_substitutes_list, argv[i], FILE_NAME_SIZE);
                file_to_be_read = false;
            } else {
                binary_name[FILE_NAME_SIZE] = '\0';
                strncpy(binary_name, argv[i], FILE_NAME_SIZE);
            }

        } // else

    } // for(int i = 1; i < argc; ++i)

    string b(binary_name);
    string n(normal);
    string s(substitute);
    string text(text_of_substitutes_list);
    size = strlen(text_of_substitutes_list);
    if(size == 0) {
        result = editRecord(n, s, b, no_searching, &which_table);
    } else {
        result = editManyRecords(text, b, no_searching, &which_table);
    }

    exit_code = result ? 0 : ERROR_CODE;
    return exit_code;
}

/*
1: MS932~EUC-JP.bin    table
2: S                  MS932
3:                   substitute
4:                     bars.txt
5: 0                   -d
6: 0                   -w
*/

void acquire_parameters_from_text(const char *parameters_text, char *binary_name, char *normal, char *substitute, char *text_of_substitutes_list, bool *no_searching, bool *which_table) {
    char *p;
    char c[C_SIZE];
    FILE *setting = fopen(parameters_text, "rt");
    int k;
    int line_number = 0;

    if(setting != NULL) {
        for(;;) {
            p = fgets(c, C_SIZE, setting);
            if(p == NULL) {
                break;
            } else {
                k = strcspn(c, "\n',|\t");
                c[k] = '\0';
            }

            line_number += 1;
            switch(line_number) {
            case 1:
                strcpy(binary_name, c); // MS932~EUC-JP.bin
                break;
            case 2:
                strcpy(normal, c); // S
                break;
            case 3:
                strcpy(substitute, c); // 
                break;
            case 4:
                strcpy(text_of_substitutes_list, c); // sortkCxx.txt
                break;
            case 5:
                if(! strcmp(DO_NOT_SEARCH_TABLE, c) ) { // -d
                    *no_searching = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *no_searching = false;
                } else {
                    *no_searching = true;
                }

                break;
            case 6:
                if(! strcmp(WHICH_TABLE, c) ) { // -w
                    *which_table = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *which_table = false;
                } else {
                    *which_table = true;
                }

                break;
            default:
                break;
            } // switch(line_number)

        } // for(;;)

        fclose(setting);
    } // if(setting != NULL)

}
