#ifndef HANZEN_H
#define HANZEN_H

#ifdef __BORLANDC__
#pragma warn -8004
#pragma warn -8012
#endif

#define VERSION 1.14
#define ASCII_BAR '_'
#define BELL 0x07
#define BELL_ASCII_KATAKANA_COUNT 224
#define BLOCK_SIZE 2560
#define BUFFER_SIZE 349568 // 128 x 2731
#define C_HANKAKU_ZENKAKU_BIN "C:\\sjisEUCjpCxx\\Hankaku~Zenkaku_katakana.bin"
#define COMMA ','
#define DAKUTEN_OFFSET 0x100
#define DECIMAL_FIELD_SIZE 512
#define ERROR_TEXT "error.txt"
#define FILE_ERROR -1L
#define FILE_ERROR_CODE -2
#define FILE_NAME_SIZE 255
#define FIRST_DAKUTEN 62
#define FIRST_HANDAKUTEN 82
#define HANDAKUTEN_OFFSET 0x200
#define HANKAKU_ZENKAKU 3 // KATAKANA_SIZE + ZENKAKU_SIZE
#define HANKAKU_ZENKAKU_BIN "Hankaku~Zenkaku_katakana.bin"
#define HANZEN_TEXT "hanzen.txt"
#define HE 0xCD // "" Shift_JIS
#define KANA_DAKUTEN 0xDE
#define KANA_HANDAKUTEN 0xDF
#define KANA_KUTEN 0xA1
#define KATAKANA_SIZE 1
#define RECORD_COUNT 86
#define SEARCH_COUNT 3
#define TEMPORARY_BINARY "binary.tmp"
#define ZENKAKU_SIZE 2
#define ZENKAKU_SIZE_1 3
#include "Dividing.h"
#include "Line.h"
#include "Numbered.h"

typedef bool (*OPEN_CLOSE_FUNCTION)(char *buffer, int buffer_size);

typedef union KANJI KANJI;
union KANJI {
    unsigned char sequence[6];
    unsigned short code[3];
    unsigned int block;
};
/*
char: 1
short: 2
int: 4
*/

class CommandPath {
    char *data;

public:
    CommandPath() {
        data = new char[BUFFER_SIZE];
        strcpy(data, getenv("Path") );
    }

    ~CommandPath() {
        delete[] data;
    }

    char *path_name(char *extra);
    std::string pathName(const char *extra);
};

class DestroyRebuild {
protected:
    bool full_house, remove_separators, way_of_arranging;
    char *buffer;
    char *combo;
    char file_name[ FILE_NAME_SIZE + 1 ];
    char new_name[ FILE_NAME_SIZE + 1 ];
    char pattern[ FIELD_COUNT + 1 ];
    int buffer_size, field_count, separator;

public:
    virtual ~DestroyRebuild() {
        delete[] buffer;
        delete[] combo;
    }

    DestroyRebuild(char *e, char *b, char *n, long z, int s, int f, bool r, bool w);
    int counting();
    int selecting();
    int selecting(int field_number, int editing);
};

class HankakuZenkaku : public DestroyRebuild {
    bool *which_table;
    bool abolish_hankaku_katakana, combining_character, no_searching, skip_control_characters;

public:
    ~HankakuZenkaku() {
    }

    HankakuZenkaku(char *e, char *b, char *n, long z, int s, int f, bool r, bool w, bool a, bool c, bool g, bool i, bool *which);
    int selecting(int field_number);
    int selecting(int field_number, char binary_extra);
};

class Standard_input {
    bool *which_table;
    bool abolish_hankaku_katakana, combining_character, no_searching, remove_separators, skip_control_characters;
    char *buffer;
    char *combo;
    char pattern[ FIELD_COUNT + 1 ];
    int buffer_size, field_count, separator;

public:
    ~Standard_input() {
        delete[] buffer;
        delete[] combo;
    }

    Standard_input(long z, int s, int f, bool r, bool a, bool c, bool g, bool i, bool *which);
    int standard_input_counting();
    int standard_input_output(int field_number);
};

class SparseKana {
    const bool DAKUTEN;
    const bool HANDAKUTEN;

    char *sparse;
    int bufferSize, size;
    unsigned char separator;

    void populated(int index, int place, unsigned char c);

public:
    char *buffer;
    char *combo;

    int kanaBufferSize() const {
        return bufferSize;
    }

    ~SparseKana() {
        delete[] buffer;
        delete[] combo;
        delete[] sparse;
    }

    bool setting(bool no_searching, bool *whichTable);
    int reading(std::string & binaryName);
    int moved(unsigned char c, bool handakuten);
    SparseKana(int b);
    unsigned char dakuten(int index, int place);
    unsigned char handakuten(int index, int place);
    unsigned char working(int index, int place);
    void clear();
};

bool binary_file_in_current_directory(char *buffer, int buffer_size);
bool binary_file_in_installed_directory(char *buffer, int buffer_size);
bool binary_file_in_path_directory(char *buffer, int buffer_size);
bool extensionEqualsCSV(char *name);
bool openClose(char *file_name);
bool seion(int beginning, char first_byte, char second_byte);
FIELD_POINTER abolish_hankaku(FIELD_POINTER p);
FIELD_POINTER hankaku_zenkaku(FIELD_POINTER p); // zenkaku.c
FIELD_POINTER replace_hankaku_with_zenkaku_except_dakuten(FIELD_POINTER p);
FIELD_POINTER skip_control_characters_to_abolish_hankaku(FIELD_POINTER p);
FIELD_POINTER skip_control_characters_to_replace_hankaku_with_zenkaku(FIELD_POINTER p); // zenkaku.c
FIELD_POINTER skip_control_characters_to_replace_hankaku_with_zenkaku_except_dakuten(FIELD_POINTER p);
FIELD_POINTER transparent(FIELD_POINTER p);
int inverted(char *c);
int punctuation(char *p, int q);
int renameBinaryFile(const char *name, bool no_removing);
void errorHandler(const char *p, char *q);
void errorHandler(const char *p, const char *q);
void versionGuide(double v, bool comments, bool loud);

#endif // HANZEN_H
