#include "keyun.h"

/*
keyun Version 1.03
Reading records to arrange lines in descending order, for example
Copyright 2019 Sogaya

gcc (GCC) 3.4.2 (mingw-special)
Borland C++ 5.5.1 for Win32

*/

extern int check_ID[LIMIT_OF_MY_MEMORY];

int main(int argc, char *argv[]) {
    using namespace std;

    const bool WITH_COMMENTS = true;
    const bool WITHOUT_COMMENTS = false;
    const char SETTING_TEXT[] = "manager.txt";
    const string EMPTY("");
    const string EUC_JP_TERM("-e");
    const string HELP("--help");
    const string NEW_FILE("-o");
    const string STANDARD_OUT("-o-");
    const string UTF_8_TERM("-8");
    const string VERSION_WITH_COMMENTS("-VC");
    const string VERSION_WITHOUT_COMMENTS("-V");

    bool add_ID = false;
    bool carriage_return = true;
    bool case_sensitive = true;
    bool descending = false;
    bool loud = true;
    bool new_file = false;
    bool remove_temporary_binary_file = true;
    bool report_ID = false;
    bool unique_summary = false;
    char file_name[ FILE_NAME_SIZE + 1 ];
    char new_name[ FILE_NAME_SIZE + 1 ];
    int exit_code = 0;
    int line_count = 0;
    int manager = PHRASES;
    int record_size;
    string *p = new string("");
    string argument;
    string fileName(TEMPORARY_BINARY);
    string newName = EMPTY;

    memset(new_name, 0, FILE_NAME_SIZE + 1);

    file_name[FILE_NAME_SIZE] = '\0';
    strncpy(file_name, fileName.c_str(), FILE_NAME_SIZE);

// Acquiring parameters from SETTING_TEXT 
    acquire_parameters_from_text(SETTING_TEXT, file_name, new_name, &line_count, &manager, &record_size, &add_ID, &case_sensitive, &report_ID, &carriage_return, &descending, &loud, &remove_temporary_binary_file, &unique_summary);
    string name(new_name);
    newName = name;

    for(int i = 1; i < argc; ++i) {
        delete p;
        p = new string(argv[i]);
        argument = *p;

        if(DESCENDING == argument) {
            descending = true;
        } else if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, argv[i]) ) {
            remove_temporary_binary_file = false;
        } else if(EUC_JP_TERM == argument) {
            carriage_return = false;
        } else if(HELP == argument) {
             delete p;
            versionGuide(VERSION, WITH_COMMENTS, loud);
        } else if(NEW_FILE == argument) {
            new_file = true;
        } else if(! strcmp(NOT_CASE_SENSITIVE, argv[i]) ) {
            case_sensitive = false;
        } else if(! strcmp(SILENT, argv[i]) ) {
            loud = false;
        } else if(STANDARD_OUT == argument) {
            newName = EMPTY;
        } else if(! strcmp(LINE_FOR_EACH_UNIQUE_SUMMARY, argv[i]) ) {
            unique_summary = true;
        } else if(UTF_8_TERM == argument) {
            carriage_return = false;
        } else if(VERSION_WITH_COMMENTS == argument) {
             delete p;
            versionGuide(VERSION, WITH_COMMENTS, loud);
        } else if(VERSION_WITHOUT_COMMENTS == argument) {
             delete p;
            versionGuide(VERSION, WITHOUT_COMMENTS, loud);
        } else {
            if(new_file) {
                newName = argument;
                new_file = false;
            } else {
                fileName = argument;
            }

        } // if

    } // for(int i = 1; i < argc; ++i)

    file_name[FILE_NAME_SIZE] = '\0';
    strncpy(file_name, fileName.c_str(), FILE_NAME_SIZE);

    new_name[FILE_NAME_SIZE] = '\0';
    strncpy(new_name, newName.c_str(), FILE_NAME_SIZE);

    if(unique_summary) {
        exit_code = arranging_unique_lines_to_produce_text(file_name, new_name, line_count, manager, record_size, add_ID, carriage_return, case_sensitive, descending);
    } else {
        exit_code = arranging_to_produce_text(file_name, new_name, line_count, manager, record_size, add_ID, carriage_return, case_sensitive, descending);
    }

    if(remove_temporary_binary_file) {
        remove(TEMPORARY_BINARY);
    }

    if(report_ID) {
        for(int h = 0; h < LIMIT_OF_MY_MEMORY && check_ID[h] > 0; ++h) {
            cout << "Line-" << check_ID[h] << endl;
        }

    }

    delete p;
    return exit_code;
}

void acquire_parameters_from_text(const char *parameters_text, char *file_name, char *new_name, int *line_count, int *manager, int *record_size, bool *add_ID, bool *case_sensitive, bool *report_ID, bool *carriage_return, bool *descending, bool *loud, bool *remove_temporary_binary_file, bool *unique_summary) {
    char *p;
    char *stop;
    char c[C_SIZE];
    FILE *setting = fopen(parameters_text, "rt");
    int k;
    int line_number = 0;

    if(setting != NULL) {
        for(;;) {
            p = fgets(c, C_SIZE, setting);
            if(p == NULL) {
                break;
            } else {
                k = strcspn(c, "\n',|\t");
                c[k] = '\0';
            }

            line_number += 1;
            switch(line_number) {
            case 1:
                *line_count = (int)strtol(p, &stop, 10); // 520 lines
                break;
            case 2:
                *manager = (int)strtol(p, &stop, 10); // 0: pharases. 1: integers. 2: decimals.
                break;
            case 3:
                *record_size = (int)strtol(p, &stop, 10); // 3072 bytes
                break;
            case 4:
                if(! strcmp(ADD_ID, c) ) { // -#
                    *add_ID = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *add_ID = false;
                } else {
                    *add_ID = true;
                }

                break;
            case 5:
                if(! strcmp(NOT_CASE_SENSITIVE, c) ) { // -f
                    *case_sensitive = false;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *case_sensitive = true;
                } else {
                    *case_sensitive = false;
                }

                break;
            case 6:
                if(! strcmp(REPORT_ID, c) ) { // -!
                    *report_ID = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *report_ID = false;
                } else {
                    *report_ID = true;
                }

                break;
            case 7:
                file_name[FILE_NAME_SIZE] = '\0';
                strncpy(file_name, c, FILE_NAME_SIZE); // binary.tmp
                break;
            case 8:
                new_name[FILE_NAME_SIZE] = '\0';
                strncpy(new_name, c, FILE_NAME_SIZE); // keyun.txt
                break;
            case 9:
                if(! strcmp(CARRIAGE_RETURN_LINE_FEED, c) ) { // -CRLF
                    *carriage_return = true;
                } else if(! strcmp(LINE_FEED, c) ) { // -LF
                    *carriage_return = false;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *carriage_return = false;
                } else {
                    *carriage_return = true;
                }

                break;
            case 10:
                if(! strcmp(DESCENDING, c) ) { // -r
                    *descending = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *descending = false;
                } else {
                    *descending = true;
                }

                break;
            case 11:
                if(! strcmp(SILENT, c) ) { // -s
                    *loud = false;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *loud = true;
                } else {
                    *loud = false;
                }

                break;
            case 12:
                if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, c) ) { // -y
                    *remove_temporary_binary_file = false;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *remove_temporary_binary_file = true;
                } else {
                    *remove_temporary_binary_file = false;
                }

                break;
            case 13:
                if(! strcmp(LINE_FOR_EACH_UNIQUE_SUMMARY, c) ) { // -u
                    *unique_summary = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *unique_summary = false;
                } else {
                    *unique_summary = true;
                }

                break;
            default:
                break;
            } // switch(line_number)

        } // for(;;)

        fclose(setting);
    } // if(setting != NULL)

}
